#ifndef DIFFUSION_MODEL_H
#define DIFFUSION_MODEL_H

#include "Common.h"
#include "Graph.h"
#include "SimpleRNG.h"

using namespace std;

class DiffusionModel{

private:

	Graph &m_G;

public:

	DiffusionModel(Graph &G):m_G(G){};
	~DiffusionModel(){};

	double EstimateNeighborhoodByIC(const set<unsigned>& sources, const unsigned &num_samples, const unsigned &T);

};

double DiffusionModel::EstimateNeighborhoodByIC(const set<unsigned>& sources, const unsigned &num_samples, const unsigned &T)
{
	SimpleRNG RNG;

	double influence = 0;
	for(unsigned iter = 0; iter < num_samples; ++ iter)
	{
		queue<unsigned> Q;
		map<unsigned, unsigned> infectedNodes;

		for(set<unsigned>::const_iterator s = sources.begin(); s != sources.end(); ++ s)
		{
			Q.push(*s);
			infectedNodes[*s] = 0;
		}

		while(!Q.empty())
		{
			unsigned u = Q.front();

			if(!m_G.nodes[u].children.empty())
			{
				unsigned global_time = infectedNodes[u];
				
				if(global_time < T)
				{
					for(set<unsigned>::const_iterator v = m_G.nodes[u].children.begin(); v != m_G.nodes[u].children.end(); ++ v)
					{
						if(infectedNodes.find(*v) == infectedNodes.end())
						{
							double toss = RNG.GetUniform();
							if(m_G.edge_weight[u][*v] >= toss)
							{
								infectedNodes[*v] = global_time + 1;
								Q.push(*v);
							}
						}
					}
				}
			}

			Q.pop();

		}

		influence += double(infectedNodes.size()) / double(num_samples);

		
	}

	return influence;
}


#endif
