function InfluLearner(network_filename_train, source_filename, num_samples, beginning, ending, N)

warning('off','MATLAB:dispatcher:nameConflict');
addpath('./minConf');
addpath('./minConf/minFunc');
addpath('./minConf/minConf');

clc;

dirname = ['synthetic','/', source_filename];

% number of random features used
K = 100;

cascade_filename_train = ['train_cascade_C', num2str(num_samples), '_', network_filename_train];

% Estimate infection probability
[P,cascade] = estimateMultiC(N, [dirname, '/', cascade_filename_train]);

inputdata = load([dirname,'/', 'train_source_C', num2str(num_samples), '_', network_filename_train,'.mat']);
allsources = inputdata.allsources';

W = zeros(K, N);

features = cell(N,1);

% beginning = 1;
% ending = N;
% trivial parallelization

for j = beginning : ending

    if(~all(P(:,j) == 0))
        
        %fprintf('%g starts\n', j);
        tic;
        
        % Generate random features
        F = rand(N, K);
        F = double(bsxfun(@lt, F, P(:,j)));
        
        % Infection label for node j
        Y_j = cascade(:,j);
        
        
        % Calculate random features
        phi = min(allsources * F,1);
        phi = phi + 1e-4;
        
        funObj = @(w)objFunction(w, phi, Y_j);
        w0 = 1 / K * ones(K,1);
        
        
        funProj = @(w)projectSimplex(w);
        options.verbose = 0; 
     
        W(:,j) = minConf_PQN(funObj, w0, funProj,options);
        
        fprintf('%g finishes %g\n', j,toc);
        
        features{j} = F;

    end

end

subdirname = [dirname,'/', 'result-', cascade_filename_train];

if(~exist(subdirname,'dir'))
   
    mkdir(subdirname);
    
end

save([subdirname, '/', 'saved-features-power-law-', num2str(beginning), '-', num2str(ending), '.mat'], 'W', 'features');
