#ifndef LEARNING_INFLUENCE_FUNCTION_H
#define LEARNING_INFLUENCE_FUNCTION_H
#include "Utility.h"

class LearningInfluenceFunction{

private:

	unsigned m_N;

	vector<vector<set<pair<double, unsigned> > > > m_node_to_cascades;

	string m_cascade_filename;

public:
    
    map<unsigned, map<unsigned, double> > m_C;
        
public:
	
	LearningInfluenceFunction(const unsigned &N, const string &cascade_filename);

	~LearningInfluenceFunction(){};

	void EstimateIndependentC(const double &T);
	
	void PrintC();

	double EstimateInfluence(const set<unsigned> &sources);

	void LoadCascade(const string &filename, bool has_post);
    
    unsigned EstimateMultiC(vector<unsigned> &row, vector<unsigned> &col);
};

LearningInfluenceFunction::LearningInfluenceFunction(const unsigned &N, const string &cascade_filename)
{
	m_N = N;

	m_node_to_cascades = vector<vector<set<pair<double, unsigned> > > > (m_N, vector<set<pair<double, unsigned> > >());

	m_cascade_filename = cascade_filename;

}

void LearningInfluenceFunction::EstimateIndependentC(const double &T)
{
	for (unsigned u = 0; u < m_N; ++ u)
	{
		if (m_node_to_cascades[u].size() > 0)
		{
			for (vector<set<pair<double, unsigned> > >::const_iterator cascade = m_node_to_cascades[u].begin(); cascade != m_node_to_cascades[u].end(); ++ cascade)
			{
				for (set<pair<double, unsigned> >::const_iterator v = cascade->begin(); v != cascade->end(); ++ v)
				{
					if ((u != v->second) && (v->first <= T))
					{
						if (m_C.find(u) == m_C.end())
						{
							map<unsigned, double> tmp;
							tmp.insert(make_pair(v->second, 1.0));

							m_C.insert(make_pair(u, tmp));
						}else{

							if (m_C[u].find(v->second) == m_C[u].end())
							{
								m_C[u].insert(make_pair(v->second, 1.0));
							}else{
								++ m_C[u][v->second];
							}
						}
					}
				}
			}
		}
	}

	for (unsigned u = 0; u < m_N; ++ u)
	{
		if (m_C.find(u) != m_C.end())
		{
			for (map<unsigned, double>::iterator m_pair = m_C[u].begin(); m_pair != m_C[u].end(); ++ m_pair)
			{
				m_pair->second /= m_node_to_cascades[u].size();
			}
		}
	}	
}

void LearningInfluenceFunction::PrintC()
{
	for (map<unsigned, map<unsigned, double> >::const_iterator row = m_C.begin(); row != m_C.end(); ++ row)
	{
		cout << row->first << " : ";
		for (map<unsigned, double>::const_iterator col = row->second.begin(); col != row->second.end(); ++ col)
		{
			cout << col->first << " " << col->second << "; ";
		}
		cout << endl;
	}
}

double LearningInfluenceFunction::EstimateInfluence(const set<unsigned> &sources)
{
	double influence = 0;
	for (unsigned v = 0; v < m_N; ++ v)
	{
		double inner_product  = 0;
		for (set<unsigned>::const_iterator s = sources.begin(); s != sources.end(); ++ s)
		{
			if ((m_C.find(*s) != m_C.end()) && (m_C[*s].find(v) != m_C[*s].end()) )
			{
				inner_product += log(1 - m_C[*s][v]);
			}
		}
		influence += (1 - exp(inner_product));
	}

	influence += sources.size();

	return influence;
}

void LearningInfluenceFunction::LoadCascade(const string &filename, bool has_post)
{
	string str;

	ifstream fin(filename.c_str(),ios::in);
	
	if(!has_post)
	{
		unsigned readline = 1;
		while(getline(fin, str))
		{
			if((readline >= m_N + 2) && (str != "")){

				vector<string> line = Utility::SeperateLineWordsVector(str, ",");
				unsigned source_node = atoi(line[0].c_str());

				set<pair<double, unsigned> > cascade;
				for(unsigned i = 0; i < line.size(); i += 2){

					unsigned nodeID = atoi(line[i].c_str());

					double infection_time = atof(line[i + 1].c_str());

					cascade.insert(make_pair(infection_time, nodeID));
					
				}

				
				m_node_to_cascades[source_node].push_back(cascade);
				
			}

			++ readline;
		}
	}else{

		while(getline(fin, str))
		{
			if((str != "")){

				vector<string> line = Utility::SeperateLineWordsVector(str, ",");
				unsigned source_node = atoi(line[0].c_str());

				set<pair<double, unsigned> > cascade;
				for(unsigned i = 0; i < line.size(); i += 3){

					unsigned nodeID = atoi(line[i].c_str());

					double infection_time = atof(line[i + 2].c_str());

					cascade.insert(make_pair(infection_time, nodeID));
					
				}

				m_node_to_cascades[source_node].push_back(cascade);
			}
		}
	}
	fin.close();
}

unsigned LearningInfluenceFunction::EstimateMultiC(vector<unsigned> &row, vector<unsigned> &col)
{
	string str;

	ifstream fin(m_cascade_filename.c_str(),ios::in);

	unsigned count = 0;

	//stringstream css,dss;

	map<unsigned, double> node_to_cascades_count;

	while(getline(fin, str))
	{
		if(str != "")
		{
			vector<string> line = Utility::SeperateLineWordsVector(str, ";");
			set<unsigned> sources;

			for(unsigned k = 0; k < line.size(); k += 3)
			{
				unsigned source = atoi(line[k].c_str());

				unsigned dest = atoi(line[k + 1].c_str());

				//css << count << ";" << dest << endl;
                row.push_back(count);
                col.push_back(dest);
                
				sources.insert(source);

				if(m_C.find(source) == m_C.end())
				{
					map<unsigned, double> tmp;
					tmp.insert(make_pair(dest, 1.0));

					m_C.insert(make_pair(source, tmp));
				}else
				{
					if(m_C[source].find(dest) == m_C[source].end())
					{
						m_C[source].insert(make_pair(dest,1.0));
					}else
					{
						++ m_C[source][dest];
					}
				}
			}

// 			copy(sources.begin(), sources.end(), ostream_iterator<unsigned, char>(dss, ";"));
// 			dss << endl;
			
			for(set<unsigned>::const_iterator s = sources.begin(); s != sources.end(); ++ s)
			{
				if(node_to_cascades_count.find(*s) == node_to_cascades_count.end())
				{
					node_to_cascades_count.insert(make_pair(*s, 1.0));
				}else
				{
					++ node_to_cascades_count[*s];
				}
			}

			++ count;
		}
	}


	fin.close();

	cout << count << " cascades." << endl;

// 	ofstream fout("sources");
// 	fout << dss.str();
// 	fout.close();

// 	fout.open("infection");
// 	fout << css.str();
// 	fout.close();


	for (unsigned u = 0; u < m_N; ++ u)
	{
		if (m_C.find(u) != m_C.end())
		{
			for (map<unsigned, double>::iterator m_pair = m_C[u].begin(); m_pair != m_C[u].end(); ++ m_pair)
			{
				m_pair->second /= node_to_cascades_count[u];
			}
		}
	}
    
    return count;

}

#endif