#include "mex.h"
#include "Common.h"
#include "LearningInfluenceFunction.h"

using namespace std;

void mexFunction(
		 int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[]
		 )
{
	unsigned N = mxGetScalar(prhs[0]);
	
	string cascade_filename_train(mxArrayToString(prhs[1]));
    
    vector<unsigned> cascade_row, cascade_col;
	
 	LearningInfluenceFunction lif(N, cascade_filename_train);
    
 	unsigned num_cascades = lif.EstimateMultiC(cascade_row, cascade_col);
    
    unsigned nzmax = 0;
    
    for(map<unsigned, map<unsigned, double> >::const_iterator row = lif.m_C.begin(); row != lif.m_C.end(); ++ row)
    {
        nzmax += row->second.size();
    }
    
    //cout << nzmax << endl;
   
    mxArray *row = mxCreateDoubleMatrix(nzmax,1,mxREAL);
    mxArray *col = mxCreateDoubleMatrix(nzmax,1,mxREAL);
    mxArray *value = mxCreateDoubleMatrix(nzmax,1,mxREAL);
    
    double *pr_row = mxGetPr(row);
    double *pr_col = mxGetPr(col);
    double *pr_value = mxGetPr(value);
    
    unsigned k = 0;
    for (map<unsigned, map<unsigned, double> >::const_iterator m_row = lif.m_C.begin(); m_row != lif.m_C.end(); ++ m_row)
    {
        for (map<unsigned, double>::const_iterator m_col = m_row->second.begin(); m_col != m_row->second.end(); ++ m_col)
        {
            pr_row[k] = m_row->first + 1;
            pr_col[k] = m_col->first + 1;
            pr_value[k] = m_col->second;
            
            ++ k;
        }
    }
    
    //cout << k << endl;
    
    mxArray *tmp[5];
    tmp[0] = row;
    tmp[1] = col;
    tmp[2] = value;
    tmp[3] = mxCreateDoubleScalar(N);
    tmp[4] = mxCreateDoubleScalar(N);
    
    mxArray *output_P[1];
    mexCallMATLAB(1, output_P, 5, tmp, "sparse");
    
    mxArray *mx_cascade_row = mxCreateDoubleMatrix(cascade_row.size(),1,mxREAL);
    mxArray *mx_cascade_col = mxCreateDoubleMatrix(cascade_col.size(),1,mxREAL);
    mxArray *mx_cascade_value = mxCreateDoubleMatrix(cascade_row.size(),1,mxREAL);
    
    double *pr_cascade_row = mxGetPr(mx_cascade_row);
    double *pr_cascade_col = mxGetPr(mx_cascade_col);
    double *pr_cascade_value = mxGetPr(mx_cascade_value);
    
    copy(cascade_row.begin(), cascade_row.end(), pr_cascade_row);
    copy(cascade_col.begin(), cascade_col.end(), pr_cascade_col);
    fill(pr_cascade_value, pr_cascade_value + cascade_row.size(), 1);
    
    for(unsigned i = 0; i < cascade_row.size(); ++ i)
    {
        ++ pr_cascade_row[i];
        ++ pr_cascade_col[i];
    }
    
    mxArray *cascade_matrix_input[5];
    cascade_matrix_input[0] = mx_cascade_row;
    cascade_matrix_input[1] = mx_cascade_col;
    cascade_matrix_input[2] = mx_cascade_value;
    cascade_matrix_input[3] = mxCreateDoubleScalar(num_cascades);
    cascade_matrix_input[4] = mxCreateDoubleScalar(N);
    
    mxArray *output_cascade_matrix[1];
    mexCallMATLAB(1, output_cascade_matrix, 5, cascade_matrix_input, "sparse");
    
    plhs[0] = output_P[0];
    plhs[1] = output_cascade_matrix[0];
    
    
}

