#include "mex.h"
#include "Common.h"
#include "SimpleRNG.h"
#include "Utility.h"

using namespace std;


void LoadCascades(const unsigned &N, const string &cascade_filename, vector<vector<set<pair<double, unsigned> > > > &node_to_cascades, bool has_post)
{
	string str;

	ifstream fin(cascade_filename.c_str(),ios::in);

	if(!has_post)
	{
		unsigned readline = 1;
		while(getline(fin, str))
		{
			if((readline >= N + 2) && (str != "")){

				vector<string> line = Utility::SeperateLineWordsVector(str, ",");
				unsigned source_node = atoi(line[0].c_str());

				set<pair<double, unsigned> > cascade;
				for(unsigned i = 0; i < line.size(); i += 2){

					unsigned nodeID = atoi(line[i].c_str());

					double infection_time = atof(line[i + 1].c_str());

					cascade.insert(make_pair(infection_time, nodeID));
					
				}

				
				node_to_cascades[source_node].push_back(cascade);
				
			}

			++ readline;
		}

	}else{

		while(getline(fin, str))
		{
			if((str != "")){

				vector<string> line = Utility::SeperateLineWordsVector(str, ",");
				unsigned source_node = atoi(line[0].c_str());

				set<pair<double, unsigned> > cascade;
				for(unsigned i = 0; i < line.size(); i += 3){

					unsigned nodeID = atoi(line[i].c_str());

					double infection_time = atof(line[i + 2].c_str());

					cascade.insert(make_pair(infection_time, nodeID));
					
				}

				node_to_cascades[source_node].push_back(cascade);
			}
		}

	}


	fin.close();

}

double EstimateRealInfluence(const vector<vector<set<pair<double, unsigned> > > > &node_to_cascades, const set<unsigned> &sources, const double &T,  SimpleRNG &RNG)
{
    unsigned samples = 1000;

	double real_influence = 0;

	for(unsigned k = 0; k < samples; ++ k)
	{
		set<unsigned> influenced_set;

		for(set<unsigned>::const_iterator s = sources.begin(); s != sources.end(); ++ s)
		{
			double random = RNG.GetUniform();

			unsigned idx = node_to_cascades[*s].size() * random;

			idx = (idx < node_to_cascades[*s].size()) ? idx : node_to_cascades[*s].size() - 1;

			for(set<pair<double, unsigned> >::const_iterator v = node_to_cascades[*s][idx].begin(); v != node_to_cascades[*s][idx].end(); ++ v)
			{
				if(v->first <= T)
				{
					influenced_set.insert(v->second);
				}
			}
		}

		real_influence += influenced_set.size();

	}

	

	return real_influence / samples;
}


void mexFunction(
		 int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[]
		 )
{
    unsigned N = mxGetScalar(prhs[0]);
	
	double T = mxGetScalar(prhs[1]);
	
	string cascade_filename_test(mxArrayToString(prhs[2]));
	
	bool *has_post = mxGetLogicals(prhs[3]);
    
    unsigned number_sources = mxGetNumberOfElements(prhs[4]);
    
    double *tmp = mxGetPr(prhs[4]);
    
    set<unsigned> sources(tmp, tmp + number_sources);
    
    vector<vector<set<pair<double, unsigned> > > > node_to_cascades(N, vector<set<pair<double, unsigned> > >());

	LoadCascades(N, cascade_filename_test, node_to_cascades, *has_post);
    
    for(set<unsigned>::const_iterator s = sources.begin(); s != sources.end(); ++ s)
    {
        if(node_to_cascades[*s].size() == 0)
        {
            cout << *s << " is not in the testing cascades." << endl;
            return;
        }
    }
    
    SimpleRNG RNG;
    
    plhs[0] = mxCreateDoubleScalar(EstimateRealInfluence(node_to_cascades, sources, T, RNG));

}